
################################################################## Date: 31Jan09 ##
## Game-Pool  version 1 beta3 by SpiKe^^       The Ultimate Tcl Game-Pool Script ##
###################################################################################
##                                                                               ##
##  Game-Pool is a customizable random team generator for online game channels.  ##
##                                                                               ##
##  Allows a user with the correct flags to open a pool for users to join the    ##
##  next online game. Messages players thier team, ip/port and password info     ##
##  when pool is full and game play can begin.                                   ##
##                                                                               ##
##   1. Run multiple game pools with this single script.                         ##
##   2. Admin & Users set by flags.                                              ##
##   3. Private & Public commands.                                               ##
##   4. Set or change the number of players for each game.                       ##
##   5. Pass Admin to another user if needed.                                    ##
##   6. Cancel Game Pool                                                         ##
##                                                                               ##
##   This is the second beta release. All feedback and suggestions are welcome.  ##
##             Contact Options to report bugs, comments & suggestions            ##
##                        http://www.mytclscripts.com                            ##
##                        irc: undernet: #pc-mIRC-help                           ##
##                       email: spike@mytclscripts.com                           ##
###################################################################################
##                                                                               ##
##                         Version 1 beta3 Release Notes                         ##
##                                                                               ##
##   1. Fixed an issue when adding/removing nicks with tcl special characters.   ##
##      Thanks to [air]traffic for finding and reporting this bug!               ##
##                                                                               ##
##   2. Added a setting to limit the number of simultaneous open game-pools.     ##
##      set gpool(pmax) ""                                                       ##
##           # limit maximum number of open game-pools? ("" = no maximum) #      ##
##                                                                               ##
###################################################################################


#########################################################################################
# Game-Pool Command Overview:  All Game-Pool Commands                                   #
#                                                                                       #
# -> Game-Pool can keep track of any number of different game-pools,                    #
#    as provided in the below setting:  gpool(games)                                    #
# -> So, in our example:     ff Fortress Forever                                        #
#                            tfc Team Fortress Classic                                  #
#                            tf2 Team Fortress 2                                        #
#    we have three different games,  represented by the game codes:  ff, tfc, & tf2     #
# -> That would make the actual !pool commands something more like:                     #
#         !poolff            :to open a Fortress Forever game-pool                      #
#         !pooltfc           :to open a Team Fortress Classic game-pool                 #
#         !pooltf2           :to open a Team Fortress 2 game-pool                       #
# -> This same system applies to all of the below Game-Pool commands.                   #
#    Some more examples:                                                                #
#         !poolff 4          :to open a Fortress Forever game-pool with 4 players       #
#         !addtfc            :to join the Team Fortress Classic game-pool               #
#         !iptf2             :see the server ip for the Team Fortress 2 game-pool       #
# -> Any command given without a game code, tries to follow the open game-pool.         #
#                                                                                       #
#########################################################################################

#########################################################################################
# Working Game-Pool Commands List                                       Version 1 beta3 #
#########################################################################################

#########################################################################################
# Open A Game-Pool (always done in the channel)  Admin Only                             #
#                                                                                       #
#  !pool       :Open a Game-Pool with the Default Number of Players,  Or...             #
#  !pool 6     :Open a Game-Pool with 6 Players                                         #
#                                                                                       #
#########################################################################################
# Private Message Game Setup (can keep login info private)  Current Pool Admin Only     #
#                                                                                       #
#  !serverip / !server / !ip    Set/Change the Server IP of the Game                    #
#                               USE: !ip <serverip:port>   EX. !ip 11.22.111.242:5524   #
#                               OR TRY: !ip <serverip:port> pass <password>             #
#                                                                                       #
#  !password / !pass            Set/Change the Server Password                          #
#                               USE: !pass <password>      EX. !pass thePassWord        #
#                               OR TRY: !pass <password> ip <serverip:port>             #
#                                                                                       #
#  !onjoin / !join              Add an On-Join Message for the Current Game-Pool        #
#                               USE: !onjoin <message>     EX. !join A short message.   #
#                               NOTE: Send this message to anyone joining the pool,     #
#                                     and to everyone already in the pool.              #
#                                                                                       #
#########################################################################################
# Public Game-Pool Setup (done in the channel)  Current Pool Admin Only                 #
#                                                                                       #
#  !pool <number>               Change the Game-Pool Size to <number> Players           #
#                               EX. !pool 10  (change the game-pool size to 10 players) #
#                                                                                       #
#  !pool server / !pool s       Set/Change the Server IP of the Game, & Make It Public  #
#  !pool ip / !pool i           USE: !ip <serverip:port>   EX. !ip 11.22.111.242:5524   #
#  !serverip / !server / !ip    OR TRY: !ip <serverip:port> pass <password>             #
#                                       to also make the server password public.        #
#                                                                                       #
#  !pool pass / !pool p         Set/Change the Server Password, & Make It Public        #
#                               USE: !pool p <password>    EX. !pool p thePassWord      #
#                                                                                       #
#  !pool quit / !pool q         Cancel the Currently Open Game-Pool                     #
#  !pool cancel / !pool c       USE: !pool quit            EX. !pool c                  #
#  !cancel / !quit              EX. !cancel                                             #
#                                                                                       #
#  !pool admin / !pool a        Give the Current Game-Pool Admin to Someone Else        #
#  !admin                       USE: !pool admin <nick>    EX. !pool a SomeNick         #
#                               EX: !admin SomeNick                                     #
#                               NOTE: New admin Must have the correct userfile flags.   #
#                                                                                       #
#  !pool remove / !pool r       Remove A Player from the Game-Pool                      #
#  !pool boot / !pool b         USE: !pool remove <nick>   EX. !pool r SomeNick         #
#  !remove / !rem               EX: !remove BadGuy                                      #
#                               OR TRY: !rem badguy somenick thug                       #
#                                                                                       #
#########################################################################################
# Advanced !pool Commands Available to the Current Pool Admin:                          #
#                                                                                       #
#   The !pool command can take almost any number of command options from above.         #
#   This makes many interresting run-on command combinations possible:                  #
#                                                                                       #
#   EX 1:  To open a pool with 6 players,  set the ip & password and make them public,  #
#          ->  !pool 6 ip 11.22.12.23:4620 pass MyPassword                              #
#                                                                                       #
#   EX 2:  We have 7 players & can't get an 8th.                                        #
#          I could remove a player & start the game with the 6 remaining players,       #
#          ->  !pool remove SomeNick 6                                                  #
#                                                                                       #
#   Important "!pool remove" Command NOTES:   The "!pool remove" command can take any   #
#   number of nicks to remove.  So in a run-on command, everything after the "remove"   #
#   option is considered a nick to be removed from the pool.  That behavior can be      #
#   interrupted by setting the pool size with a number, as in EX 2 above.  Or make sure #
#   to always put the remove option last if it won't be closed with a number, see EX 3. #
#                                                                                       #
#   EX 3:  Set the server ip and remove two nicks from the pool,                        #
#          ->  !pool ip 11.22.12.23:4620 remove badguy somenick                         #
#                                                                                       #
#########################################################################################
# Emergency Take Game-Pool Admin Command (always done in the channel)  Admin Only       #
#                                                                                       #
#  !admin -me / !admin +me      Take Game-Pool Admin if Current Admin Gets Lost!        #
#                               USE: !admin -me            EX. !admin -me               #
#                               OR TRY: !admin -me quit    To Cancel an Abandoned Pool  #
#                                                                                       #
#########################################################################################

#########################################################################################
# Public Game Commands (done in the channel)  Admin & Player                            #
#                                                                                       #
#  !add                         Add Yourself to the Current Game-Pool                   #
#                               USE: !add                  EX. !add                     #
#                                                                                       #
#  !remove / !rem               Remove Yourself from the Current Game-Pool              #
#                               USE: !remove               EX. !rem                     #
#                                                                                       #
#  !serverip / !server / !ip    Show the Server IP to the Channel                       #
#                               USE: !serverip             EX. !ip                      #
#                               NOTE: If done by the admin, this makes the ip public.   #
#                                                                                       #
#########################################################################################


###################################################################################
#####   SCRIPT SETTINGS   #####   SCRIPT SETTINGS   #####   SCRIPT SETTINGS   #####
###################################################################################

set gpool(chan) "#YourChannel"    ;# set 1 channel to run the script in #

set gpool(admin) "o|o"    ;# flags for admin #
set gpool(player) "-|-"   ;# flags to join a pool #

set gpool(dsize) "8"   ;# default number of players #

set gpool(pubpre) "!"   ;# command prefix for public commands (can be "") #
set gpool(msgpre) "!"   ;# command prefix for private message commands (can be "") #

## maximum number of open game-pools (up to 1 per game listed in gpool(games) below) ##
set gpool(pmax) ""    ;# limit maximum number of open game-pools? ("" = no maximum) #

## list the games the channel does pools for ##
## the first word is the game code used in all game-pool commands ##
## everything else is the name of the game ##
set gpool(games) {

ff Fortress Forever
tfc Team Fortress Classic
tf2 Team Fortress 2

}

####################################################################################
####  END SETTINGS  ####  END SETTINGS  ####  END SETTINGS  ####  END SETTINGS  ####
####################################################################################

## first public output:  A Fortress Forever pool has been started by o2! ##
## %g = game ## %a = admin ## %p = gpool(pubpre) ## %s = short game code ##
set gpool(pub1) {

14A 8,4%g 14pool has been started by 7%a14!
14To join this pool type: 7%padd%s

}

## game info message to players:  You are on team 1. Server: <ip:port> Password: <pass> ....##
## %g = game ## %a = admin ## %p = gpool(pubpre) ## %s = short game code
## %t = team number ## %i = ip ## %w = password ##
set gpool(msg1) {

You are on team7 %t14.
14Server:12 %i 14Password:12 %w
14IRC Launcher: 12ff://%i/%w
14Console command: 12password %w; connect %i

}

####################################################################################
####  END SETTINGS  ####  END SETTINGS  ####  END SETTINGS  ####  END SETTINGS  ####
####################################################################################

proc GIsOdd {str} {
 if {[string match {[13579]} [string index $str end]]} {  return 1  }
 return 0
}
if {[GIsOdd $gpool(dsize)]} {  incr gpool(dsize)  }

bind pubm $gpool(admin) "$gpool(chan) $gpool(pubpre)admin*" GAdmin
proc GAdmin {nk uh hn ch tx} {  global gpool
 if {$gpool(pubpre) ne ""} {  set tx [string trimleft $tx $gpool(pubpre)]  }
 set tx [split $tx]  ;  set cmd [string replace [lindex $tx 0] 0 4]
 set gls [GIsAdmin $nk]  ;  set als [GActivPools]
 if {$cmd eq ""} {
   if {$gls ne ""} {  set gm [lindex $gls 0]  } elseif {$als ne ""} {  set gm [lindex $als 0]
   } else {  set gm [lindex $gpool(-gindex) 0]  }
 } elseif {[lsearch -exact $gpool(-gindex) $cmd]>"-1"} {  set gm $cmd
 } else {  return 0  }
 set pre "6\[[string toupper $gm]-Pool\]"
 if {$gpool(-admin$gm) eq ""} {  puthelp "PRIVMSG $ch :$pre14 No pickup at the moment."
 } elseif {[llength $tx]=="1"} {  puthelp "PRIVMSG $ch :$pre14 Current Admin: $gpool(-admin$gm)"  
 } elseif {$gpool(-admin$gm) ne $nk} {
   if {[lindex $tx 1] eq "+me" || [lindex $tx 1] eq "-me"} {  set gpool(-admin$gm) $nk
     puthelp "PRIVMSG $ch :$pre14 Admin changed to: 7$nk"
     if {[llength $tx]>"2"} {  set tx "pool$gm [join [lrange $tx 2 end]]"  ;  GPool $nk $uh $hn $ch $tx  }
   } else {  puthelp "PRIVMSG $ch :$pre14 Current Admin: $gpool(-admin$gm)"  }
 } else {  set tx "pool$gm a [join [lrange $tx 1 end]]"  ;  GPool $nk $uh $hn $ch $tx  }
}

bind pubm $gpool(admin) "$gpool(chan) $gpool(pubpre)cancel*" GQuit
bind pubm $gpool(admin) "$gpool(chan) $gpool(pubpre)quit*" GQuit
proc GQuit {nk uh hn ch tx} {  global gpool
 if {$gpool(pubpre) ne ""} {  set tx [string trimleft $tx $gpool(pubpre)]  }
 set tx [split $tx]  ;  set cmd [lindex $tx 0]
 if {[string match -nocase cancel* $cmd]} {  set cmd [string replace $cmd 0 5]
 } elseif {[string match -nocase quit* $cmd]} {  set cmd [string replace $cmd 0 3]  }
 set gls [GIsAdmin $nk]  ;  set als [GActivPools]
 if {$cmd eq ""} {
   if {$gls ne ""} {  set gm [lindex $gls 0]  } elseif {$als ne ""} {  set gm [lindex $als 0]
   } else {  set gm [lindex $gpool(-gindex) 0]  }
 } elseif {[lsearch -exact $gpool(-gindex) $cmd]>"-1"} {  set gm $cmd
 } else {  return 0  }
 set pre "6\[[string toupper $gm]-Pool\]"
 if {$gpool(-admin$gm) ne $nk} {
   if {$gpool(-admin$gm) eq ""} {  puthelp "PRIVMSG $ch :$pre14 No pickup at the moment."
   } else {  puthelp "PRIVMSG $ch :$pre14 Current Admin: $gpool(-admin$gm)"  }
   return 0
 }
 GClear $gm
 puthelp "PRIVMSG $ch :$pre14 The game was cancelled by 7$nk14."
}

proc GClear {gm} {  global gpool  ;  set gpool(-admin$gm) ""
 foreach temp {-psz -pls -onj -srvr -pass -pubip} {  unset gpool($temp$gm)  }
}

bind msgm $gpool(admin) $gpool(msgpre)join* GJoinMsg
bind msgm $gpool(admin) $gpool(msgpre)onjoin* GJoinMsg
proc GJoinMsg {nk uh hn tx} {  global gpool
 if {$gpool(msgpre) ne ""} {  set tx [string trimleft $tx $gpool(msgpre)]  }
 set tx [split $tx]  ;  set cmd [lindex $tx 0]
 if {[string match -nocase onjoin* $cmd]} {  set cmd [string replace $cmd 0 5]
 } elseif {[string match -nocase join* $cmd]} {  set cmd [string replace $cmd 0 3]  }
 set gls [GIsAdmin $nk]  ;  set als [GActivPools]
 if {$cmd eq ""} {
   if {$gls ne ""} {  set gm [lindex $gls 0]  } elseif {$als ne ""} {  set gm [lindex $als 0]
   } else {  set gm [lindex $gpool(-gindex) 0]  }
 } elseif {[lsearch -exact $gpool(-gindex) $cmd]>"-1"} {  set gm $cmd
 } else {  puthelp "PRIVMSG $nk :10\[Game-Pool\]14 Unknown Game Code: $cmd"  ;  return 0  }
 set pre "6\[[string toupper $gm]-Pool\]"
 if {$gpool(-admin$gm) ne $nk} {
   if {$gpool(-admin$gm) eq ""} {  puthelp "PRIVMSG $nk :$pre14 No pickup at the moment."
   } else {  puthelp "PRIVMSG $nk :$pre14 Current Admin: $gpool(-admin$gm)"  }
   return 0
 }
 set tx [join [lrange $tx 1 end]]  ;  lappend gpool(-onj$gm) $tx
 puthelp "PRIVMSG $nk :$pre14 On-Join Message added."
 if {$gpool(-pls$gm) ne ""} {
   foreach nik $gpool(-pls$gm) {
    if {$nik ne $nk} {  puthelp "PRIVMSG $nik :$pre14 Message from $nk: $tx"  }
   }
 }
}

proc GActivPools {} {  global gpool  ;  set als ""
 foreach game $gpool(-gindex) {  if {$gpool(-admin$game) ne ""} { lappend als $game }  }
 return $als
}
proc GIsAdmin {nk} {  global gpool  ;  set als ""
 foreach game $gpool(-gindex) {  if {$gpool(-admin$game) eq $nk} { lappend als $game }  }
 return $als
}
proc GWaitin {nk} {  global gpool  ;  set wls ""
 foreach gm $gpool(-gindex) {
  if {$gpool(-info$gm) ne "" && [lindex $gpool(-info$gm) 0] eq $nk} {
    foreach {adm pls ip pas} $gpool(-info$gm) {  break  }
    if {$ip eq "" || $pas eq ""} {  lappend wls $gm  }
  }
 }
 return $wls
}
proc GFullName {gm} {  global gpool  ;  set gfull ""
 if {[set idx [lsearch -exact $gpool(-gindex) $gm]]>"-1"} {  set gfull [lindex $gpool(-gfull) $idx]  }
 return $gfull
}

bind pubm $gpool(player) "$gpool(chan) $gpool(pubpre)server*" GPubIp
bind pubm $gpool(player) "$gpool(chan) $gpool(pubpre)ip*" GPubIp
proc GPubIp {nk uh hn ch tx} {  global gpool
 if {$gpool(pubpre) ne ""} {  set tx [string trimleft $tx $gpool(pubpre)]  }
 GSetGame $nk $uh $hn $tx 1
}

bind msgm $gpool(admin) $gpool(msgpre)server* GSetGame
bind msgm $gpool(admin) $gpool(msgpre)ip* GSetGame
bind msgm $gpool(admin) $gpool(msgpre)pass* GSetGame
proc GSetGame {nk uh hn tx {wat 0} } {  global gpool
 if {$wat=="0" && $gpool(msgpre) ne ""} {  set tx [string trimleft $tx $gpool(msgpre)]  }
 set tx [split $tx]  ;  set cmd [lindex $tx 0]
 if {[string match -nocase serverip* $cmd]} {  set is ip  ;  set cmd [string replace $cmd 0 7]
 } elseif {[string match -nocase server* $cmd]} {  set is ip  ;  set cmd [string replace $cmd 0 5]
 } elseif {[string match -nocase ip* $cmd]} {  set is ip  ;  set cmd [string replace $cmd 0 1]
 } elseif {[string match -nocase password* $cmd]} {  set is pass  ;  set cmd [string replace $cmd 0 7]
 } elseif {[string match -nocase pass* $cmd]} {  set is pass  ;  set cmd [string replace $cmd 0 3]  }
 set wls [GWaitin $nk]  ;  set gls [GIsAdmin $nk]  ;  set als [GActivPools]
 if {$cmd eq ""} {
   if {$wls ne ""} {  set gm [lindex $wls 0]  } elseif {$gls ne ""} {  set gm [lindex $gls 0]
   } elseif {$als ne ""} {  set gm [lindex $als 0]  } else {  set gm [lindex $gpool(-gindex) 0]  }
 } elseif {[lsearch -exact $gpool(-gindex) $cmd]>"-1"} {  set gm $cmd
 } else {
   if {$wat=="0"} {  puthelp "PRIVMSG $nk :10\[Game-Pool\]14 Unknown Game Code: $cmd"  }  ;  return 0
 }
 set pre "6\[[string toupper $gm]-Pool\]"
 if {[llength $tx]=="1"} {
   if {$wat=="0"} {
     if {[lsearch -exact $wls $gm]>"-1"} {
       foreach {adm pls ip pas onj savd} $gpool(-info$gm) {  break  }
       if {$is eq "ip" && $ip ne ""} {  puthelp "PRIVMSG $nk :$pre14 Server IP: $ip"  }
       if {$is eq "pass" && $pas ne ""} {  puthelp "PRIVMSG $nk :$pre14 Password: $pas"  }
       if {$ip eq ""} {
         puthelp "PRIVMSG $nk :$pre14 Server IP Not Set. Set it now! Use: $gpool(msgpre)ip$gm <ServerIP:Port>"  }
       if {$pas eq ""} {
         puthelp "PRIVMSG $nk :$pre14 Password Not Set. Set it now! Use: $gpool(msgpre)pass$gm <Password>"  }
     } elseif {$gpool(-admin$gm) eq ""} {
       puthelp "PRIVMSG $nk :$pre14 No pickup at the moment."
     } elseif {$gpool(-admin$gm) ne $nk} {
       puthelp "PRIVMSG $nk :$pre14 Current Admin: $gpool(-admin$gm)"
     } elseif {$is eq "ip" && $gpool(-srvr$gm) eq ""} {
       puthelp "PRIVMSG $nk :$pre14 Server IP Not Set. To set it now, Use: $gpool(msgpre)[lindex $tx 0] <ServerIP:Port>"
     } elseif {$is eq "ip"} {
       puthelp "PRIVMSG $nk :$pre14 Server IP: $gpool(-srvr$gm)"
     } elseif {$is eq "pass" && $gpool(-pass$gm) eq ""} {
       puthelp "PRIVMSG $nk :$pre14 Password Not Set. To set it now, Use: $gpool(msgpre)[lindex $tx 0] <Password>"
     } else {
       puthelp "PRIVMSG $nk :$pre14 Password: $gpool(-pass$gm)"
     }
   } elseif {$wat=="1"} {
     if {$gpool(-admin$gm) eq ""} {  return 0
     } elseif {$gpool(-pubip$gm)>"0"} {
       if {$gpool(-srvr$gm) ne ""} {  set put "Server IP:7 $gpool(-srvr$gm)"
         if {$gpool(-pubip$gm)>"1"} {  append put " 14 Password:7 $gpool(-pass$gm)"  }
         puthelp "PRIVMSG $gpool(chan) :$pre14 $put"
       } elseif {$gpool(-admin$gm) eq $nk} {
         puthelp "PRIVMSG $gpool(chan) :$pre14 Server IP undefined - use: $gpool(pubpre)[lindex $tx 0] <ServerIP:Port>"
       } elseif {[lsearch -exact $gpool(-pls$gm) $nk]>"-1"} {
         puthelp "PRIVMSG $gpool(chan) :$pre14 Server IP undefined - pm your admin."
       }
     } else {
       if {$gpool(-admin$gm) eq $nk} {  set gpool(-pubip$gm) 1
         if {$gpool(-srvr$gm) eq ""} {  puthelp "PRIVMSG $gpool(chan) :$pre14 Server IP undefined."
         } else {  puthelp "PRIVMSG $gpool(chan) :$pre14 Server IP:7 $gpool(-srvr$gm)"  }
       } elseif {[lsearch -exact $gpool(-pls$gm) $nk]>"-1"} {
         if {$gpool(-srvr$gm) eq ""} {  puthelp "PRIVMSG $gpool(chan) :$pre14 Server IP undefined - pm your admin."
         } else {  puthelp "PRIVMSG $gpool(chan) :$pre14 Server IP not public - pm your admin."  }
       }
     }
   }
   return 0
 }
 if {[lsearch -exact $wls $gm]>"-1"} {  set doin 1  ;  set dopub 0
   foreach {adm pls ip pas onj savd} $gpool(-info$gm) {  break  }
 } elseif {$gpool(-admin$gm) eq $nk} {  set doin 0
   set ip $gpool(-srvr$gm)  ;  set pas $gpool(-pass$gm)
 } else {
   if {$gpool(-admin$gm) eq "" && $wat=="0"} {  puthelp "PRIVMSG $nk :$pre14 No pickup at the moment."
   } elseif {$wat=="0"} {  puthelp "PRIVMSG $nk :$pre14 Current Admin: $gpool(-admin$gm)"  }
   return 0
 }
 if {$is eq "pass"} {
   if {$pas eq [lindex $tx 1]} {  set didpass is
   } elseif {$pas eq ""} {  set didpass "set to"  } else {  set didpass "changed to"  }
   if {$doin=="0"} {  set gpool(-pass$gm) [lindex $tx 1]  }
   set pas [lindex $tx 1]
   if {[llength $tx]>"3"} {  set tmp [lindex $tx 2]
     if {$gpool(msgpre) ne ""} {  set tmp [string trimleft $tmp $gpool(msgpre)]  }
     if {[string match -nocase serv* $tmp] || [string match -nocase ip* $tmp]} {
       if {$ip eq [lindex $tx 3]} {  set didip is
       } elseif {$ip eq ""} {  set didip "set to"  } else {  set didip "changed to"  }
       if {$doin=="0"} {  set gpool(-srvr$gm) [lindex $tx 3]  }
       set ip [lindex $tx 3]
     }
   }
 } else {
   if {$ip eq [lindex $tx 1]} {  set didip is
   } elseif {$ip eq ""} {  set didip "set to"  } else {  set didip "changed to"  }
   if {$doin=="0"} {  
     if {$wat=="1" && $gpool(-pubip$gm)=="0"} {  incr gpool(-pubip$gm)  }
   } elseif {$wat=="1"} {  set dopub 1  }
   if {$doin=="0"} {  set gpool(-srvr$gm) [lindex $tx 1]  }
   set ip [lindex $tx 1]
   if {[llength $tx]>"3"} {  set tmp [lindex $tx 2]
     if {$wat=="0" && $gpool(msgpre) ne ""} {  set tmp [string trimleft $tmp $gpool(msgpre)]  }
     if {$wat=="1" && $gpool(pubpre) ne ""} {  set tmp [string trimleft $tmp $gpool(pubpre)]  }
     if {[string match -nocase pass* $tmp]} {
       if {$pas eq [lindex $tx 3]} {  set didpass is
       } elseif {$pas eq ""} {  set didpass "set to"  } else {  set didpass "changed to"  }
       if {$doin=="0"} {  set gpool(-pass$gm) [lindex $tx 3]
         if {$wat=="1" && $gpool(-pubip$gm)=="1"} {  incr gpool(-pubip$gm)  }
       } elseif {$wat=="1"} {  set dopub 2  }
       set pas [lindex $tx 3]
     }
   }
 }
 if {$doin=="0"} {  set dopub $gpool(-pubip$gm)  }
 if {[info exists didip] && $dopub>"0"} {  set put "Server IP:7 $ip"
   if {$wat=="0" && $didip eq "is"} {  unset put  } else {
     if {$dopub>"1"} {  append put " 14 Password:7 $pas"  }
     puthelp "PRIVMSG $gpool(chan) :$pre14 $put"
   }
 }
 if {$wat=="0"} {
   if {[info exists didip] && [info exists didpass]} {
     if {$didip eq "is" && $didpass ne "is"} {
       puthelp "PRIVMSG $nk :$pre14 Password $didpass: $pas and Server IP is: $ip"
     } else {
       puthelp "PRIVMSG $nk :$pre14 Server IP $didip: $ip and Password $didpass: $pas"
     }
   } elseif {[info exists didip]} {  puthelp "PRIVMSG $nk :$pre14 Server IP $didip: $ip"
   } elseif {[info exists didpass]} {  puthelp "PRIVMSG $nk :$pre14 Password $didpass: $pas"  }
 }
 if {$doin=="1"} {  set gpool(-info$gm) [list $adm $pls $ip $pas $onj $savd]
   if {$ip ne "" && $pas ne ""} {  GSendInfo $gm 1
   } else {
     if {$ip eq ""} {
       puthelp "PRIVMSG $nk :$pre14 Server IP Not Set. Set it now! Use: $gpool(msgpre)ip$gm <ServerIP:Port>"  }
     if {$pas eq ""} {
       puthelp "PRIVMSG $nk :$pre14 Password Not Set. Set it now! Use: $gpool(msgpre)pass$gm <Password>"  }
   }
 }
}

foreach gtmp(tmp) {games pub1 msg1} {  set gtmp(tm2) $gpool($gtmp(tmp))
 set gtmp(tm2) [split $gtmp(tm2) "\n"]  ;  set gtmp(tls) ""
 foreach gtmp(tm3) $gtmp(tm2) {
  if {$gtmp(tm3) ne "" && $gtmp(tm3) ne " "} { lappend gtmp(tls) $gtmp(tm3) }
 }
 set gpool($gtmp(tmp)) $gtmp(tls)
}
set gpool(-gindex) ""  ;  set gpool(-gfull) ""
foreach gtmp(tmp) $gpool(games) {  set gtmp(tmp) [split $gtmp(tmp)]
 lappend gpool(-gindex) [lindex $gtmp(tmp) 0]
 lappend gpool(-gfull) [join [lrange $gtmp(tmp) 1 end]]
}
foreach gtmp(tmp) $gpool(-gindex) {
 if {![info exists gpool(-admin$gtmp(tmp))]} {  set gpool(-admin$gtmp(tmp)) ""
   set gpool(-info$gtmp(tmp)) ""  ;  set gpool(-hist$gtmp(tmp)) ""
 }
}
if {![string is digit -strict $gpool(pmax)]} { set gpool(pmax) [llength $gpool(games)] }
if {$gpool(pmax)<"1"} {  set gpool(pmax) 1  }

bind pubm $gpool(admin) "$gpool(chan) $gpool(pubpre)pool*" GPool
proc GPool {nk uh hn ch tx} {  global gpool
 if {$gpool(pubpre) ne ""} {  set tx [string trimleft $tx $gpool(pubpre)]  }
 set tx [split $tx]  ;  set cmd [string replace [lindex $tx 0] 0 3]
 set gls [GIsAdmin $nk]  ;  set als [GActivPools]
 if {$cmd eq ""} {
   if {$gls ne ""} {  set gm [lindex $gls 0]
   } elseif {$als ne ""} {  set gm [lindex $als 0]
   } else {  set gm [lindex $gpool(-gindex) 0]  }
 } elseif {[lsearch -exact $gpool(-gindex) $cmd]>"-1"} {  set gm $cmd
 } else {  return 0  }
 set pre "6\[[string toupper $gm]-Pool\]"
 if {$gpool(-admin$gm) ne "" && $gpool(-admin$gm) ne $nk} {
     puthelp "PRIVMSG $ch :$pre14 Current Admin: $gpool(-admin$gm)"  ;  return 0  }
 set do ""  ;  if {[llength $tx]>"1"} {  set do [lrange $tx 1 end]  }
 if {$gpool(-admin$gm) eq ""} {
   if {[llength $als]>=$gpool(pmax)} {
     foreach x $als {  set pre "6\[[string toupper $x]-Pool\]14 Player pool open."
      if {[set y $gpool(-admin$x)] ne $nk} {
        puthelp "PRIVMSG $ch :$pre Current Admin: $y"
      } else {  puthelp "PRIVMSG $ch :$pre You are the current admin."  }
     }
     return 0
   }
   set gpool(-admin$gm) $nk  ;  set gpool(-psz$gm) $gpool(dsize)
   set gpool(-pls$gm) ""  ;  set gpool(-onj$gm) ""  ;  set gpool(-srvr$gm) ""  ;  set gpool(-pass$gm) ""
   set gpool(-pubip$gm) 0
   if {$do ne ""} {
     if {[string is digit -strict [lindex $do 0]]} {
         set gpool(-psz$gm) [lindex $do 0]  ;  set do [lreplace $do 0 0]  }
     if {$gpool(-psz$gm)=="0"} {  set gpool(-psz$gm) $gpool(dsize)  }
     if {[GIsOdd $gpool(-psz$gm)]} {  incr gpool(-psz$gm)  }
   }
   foreach temp $gpool(pub1) {
    puthelp "PRIVMSG $ch :[string map [list %g [GFullName $gm] %a $gpool(-admin$gm) %p $gpool(pubpre) %s $gm] $temp]"
   }
   puthelp "PRIVMSG $nk :4[GFullName $gm]14 pool started.  Use message commands 7$gpool(msgpre)onjoin$gm14, 7$gpool(msgpre)server$gm14 & 7$gpool(msgpre)pass$gm14 to set up this game."
   GShoPool $gm
 } elseif {[llength $tx]=="1"} {
   puthelp "PRIVMSG $ch :$pre14 Player pool open. You are the current admin."
 }
 if {$do ne ""} {  set didip ""  ;  set didpas ""  ;  set newadm ""
   while {$do ne ""} {  set temp [lindex $do 0]  ;  set t1 [string index $temp 0]
    if {[string is digit -strict $temp]} {  set do [lreplace $do 0 0]
      if {$temp!=$gpool(-psz$gm) && $temp!="0"} {
        if {$temp<[llength $gpool(-pls$gm)]} {  set temp [llength $gpool(-pls$gm)]  }
        if {[GIsOdd $temp]} {  incr temp  }
        set gpool(-psz$gm) $temp  ;  GShoPool $gm
      }
    } elseif {$t1 eq "r" || $t1 eq "b"} {
      if {[llength $do]>"1"} {  set tmp2 [lrange $do 1 end]  ;  set idx 0  ;  set end end
        foreach item $tmp2 {
         if {[string is digit -strict $item]} {  set end $idx  ;  break  }
         incr idx
        }
        if {$end eq "0"} {  set tmp2 $nk  ;  set do [lreplace $do 0 0]
        } elseif {$end ne "end"} {  set tmp2 [lrange $do 1 $end]  ;  set do [lreplace $do 0 $end]
        } else {  set do ""  }
      } else {  set tmp2 $nk  ;  set do ""  }
      set boot ""  ;  set not ""
      foreach nik $tmp2 {
       if {[set fnd [lsearch -exact [string tolower $gpool(-pls$gm)] [string tolower $nik]]]>"-1"} {
         set rnik [lindex $gpool(-pls$gm) $fnd]
       } elseif {[set fnd [lsearch [string tolower $gpool(-pls$gm)] [string tolower $nik*]]]>"-1"} {
         set rnik [lindex $gpool(-pls$gm) $fnd]
       }
       if {$fnd=="-1"} {  lappend not $nik
       } else {  set gpool(-pls$gm) [lreplace $gpool(-pls$gm) $fnd $fnd]  ;  lappend boot $rnik
         if {$gpool(-pls$gm) eq ""} {  break  }
       }
      }
      if {$not ne ""} {
        if {[llength $not]=="1"} {  puthelp "PRIVMSG $ch :$pre14 [join $not] isn't in the pool."
        } else {  puthelp "PRIVMSG $ch :$pre14 [join [lrange $not 0 end-1] ", "] & [lindex $not end] aren't in the pool."  }
      }
      if {$boot ne ""} {
        if {[llength $boot]=="1"} {  puthelp "PRIVMSG $ch :$pre14 Removed [join $boot] from the pool."
        } else { puthelp "PRIVMSG $ch :$pre14 Removed [join [lrange $boot 0 end-1] ", "] & [lindex $boot end] from the pool." }
      }
    } elseif {$t1 eq "c" || $t1 eq "q"} {
      if {[llength $do]=="1"} {  GQuit $nk $uh $hn $ch quit$gm  ;  return 0
      } else {  break  }
    } elseif {$t1 eq "s" || $t1 eq "i"} {
      if {[llength $do]>"1"} {  set didip [lindex $do 1]  ;  set do [lreplace $do 0 1]
      } else {  set didip #SHOW#  ;  break  }
    } elseif {$t1 eq "p"} {
      if {[llength $do]>"1"} {  set didpas [lindex $do 1]  ;  set do [lreplace $do 0 1]
      } else {  set didpas #SHOW#  ;  break  }
    } elseif {$t1 eq "a"} {
      if {[llength $do]>"1"} {  set nik [lindex $do 1]  ;  set do [lreplace $do 0 1]
        set newadm ""  ;  set cant ""
        if {![onchan $nik $ch]} {  puthelp "PRIVMSG $ch :$pre14 $nik isn't on $ch"
        } else {  set han [nick2hand $nik $ch]
          if {$han eq "*"} {  set han ""  }
          if {[string match -* $gpool(admin)] || $han eq ""} {  set cls [chanlist $ch]
            set realnk [lsearch -exact [string tolower $cls] [string tolower $nik]]
            if {$realnk>"-1"} {  set realnk [lindex $cls $realnk]  } else {  set realnk $nik  }
          }
          if {[string match -* $gpool(admin)]} {  set newadm $realnk
          } elseif {$han eq ""} {  set cant $realnk
          } else {  set realnk [hand2nick $han $ch]
            if {[string first | $gpool(admin)]=="-1"} {  set ok [matchattr $han $gpool(admin)]
            } else {  set ok [matchattr $han $gpool(admin) $ch]  }
            if {$ok=="1"} {  set newadm $realnk  } else {  set cant $realnk  }
          }
        }
        if {$cant ne ""} {
          puthelp "PRIVMSG $ch :$pre14 $cant doesn't have the correct userfile flags to be an admin."  
        }
      } else {  break  }
    }
   }
   if {$didip ne "" || $didpas ne ""} {
     if {$didpas ne ""} {  set gpool(-pubip$gm) 2
     } elseif {$didip ne "" && $gpool(-pubip$gm)=="0"} {  incr gpool(-pubip$gm)  }
     if {$didip eq "#SHOW#" && $didpas eq ""} {  GSetGame $nk $uh $hn ip$gm 1
     } elseif {$didip eq "" && $didpas eq "#SHOW#"} {
       if {$gpool(-srvr$gm) ne "" && $gpool(-pass$gm) ne ""} {  GSetGame $nk $uh $hn ip$gm 1
       } elseif {$gpool(-pass$gm) ne ""} {  puthelp "PRIVMSG $ch :$pre14 Password:7 $gpool(-pass$gm)"
       } else {  puthelp "PRIVMSG $ch :$pre14 Password undefined - use: $gpool(pubpre)pool$gm pass <GamePasswoed>"  }
     } else {  set send ""
       if {$didip ne ""} {
         if {$didip eq "#SHOW#" && $gpool(-srvr$gm) ne ""} {  set didip $gpool(-srvr$gm)  }
         if {$didip ne "#SHOW#"} {  set send "ip$gm $didip"  }
       }
       if {$didpas ne ""} {
         if {$didpas eq "#SHOW#" && $gpool(-pass$gm) ne ""} {  set didpas $gpool(-pass$gm)  }
         if {$didpas ne "#SHOW#"} {
           if {$send ne ""} {  append send " pass $didpas"
           } else {  set gpool(-pass$gm) $didpas  ;  puthelp "PRIVMSG $ch :$pre14 Password:7 $didpas"  }
         }
       }
       if {$send ne ""} {  GSetGame $nk $uh $hn $send 1  }
       if {$didip eq "#SHOW#"} {  GSetGame $nk $uh $hn ip$gm 1  }
       if {$didpas eq "#SHOW#"} {
         puthelp "PRIVMSG $ch :$pre14 Password undefined - use: $gpool(pubpre)pool$gm pass <GamePasswoed>"
       }
     }
   }
   if {$newadm ne ""} {  set gpool(-admin$gm) $newadm
     puthelp "PRIVMSG $ch :$pre14 Admin changed to: 7$newadm"  
   }
   if {[llength $gpool(-pls$gm)]==$gpool(-psz$gm)} {  GStart $gm  }
 }
}

bind pubm $gpool(player) "$gpool(chan) $gpool(pubpre)add*" GAddRem
bind pubm $gpool(player) "$gpool(chan) $gpool(pubpre)rem*" GAddRem
proc GAddRem {nk uh hn ch tx} {  global gpool  ;  set als [GActivPools]
 if {$gpool(pubpre) ne ""} {  set tx [string trimleft $tx $gpool(pubpre)]  }
 set tx [split $tx]  ;  set cmd [lindex $tx 0]
 if {$als eq ""} {
   if {[string match add* $cmd]} {  puthelp "PRIVMSG $ch :10\[Game-Pool\]14 No pickup at the moment."  }
   return 0
 }
 if {[string match -nocase add* $cmd]} {  set is ad  ;  set cmd [string replace $cmd 0 2]
 } elseif {[string match -nocase remove* $cmd]} {  set is rm  ;  set cmd [string replace $cmd 0 5]
 } elseif {[string match -nocase rem* $cmd]} {  set is rm  ;  set cmd [string replace $cmd 0 2]  }
 set gls [GIsAdmin $nk]
 if {$cmd eq ""} {
   if {$gls ne ""} {  set gm [lindex $gls 0]  } elseif {$als ne ""} {  set gm [lindex $als 0]
   } else {  set gm [lindex $gpool(-gindex) 0]  }
 } elseif {[lsearch -exact $gpool(-gindex) $cmd]>"-1"} {  set gm $cmd
 } else {
   if {$is eq "ad"} {  puthelp "PRIVMSG $ch :10\[Game-Pool\]14 Unknown Game Code: $cmd"  }  ;  return 0
 }
 set pre "6\[[string toupper $gm]-Pool\]"
 if {$gpool(-admin$gm) eq ""} {
   if {$is eq "ad"} {  puthelp "PRIVMSG $ch :$pre14 No pickup at the moment."  }  ;  return 0
 }
 if {$is eq "ad"} {
   if {$gpool(-pls$gm) eq ""} {  lappend gpool(-pls$gm) $nk  ;  GShoPool $gm
   } else {
     if {[lsearch -exact $gpool(-pls$gm) $nk]>"-1"} {
       return 0
     } else {  lappend gpool(-pls$gm) $nk  ;  GShoPool $gm  }
   }
   if {$gpool(-onj$gm) ne "" && $nk ne $gpool(-admin$gm)} {
     foreach ln $gpool(-onj$gm) {  puthelp "PRIVMSG $nk :$pre14 Message from $gpool(-admin$gm): $ln"  }
   }
   if {[llength $gpool(-pls$gm)]==$gpool(-psz$gm)} {  GStart $gm  }
 } else {
   if {$nk == $gpool(-admin$gm) && [llength $tx]>"1"} {
     set tx "pool$gm r [join [lrange $tx 1 end]]"  ;  GPool $nk $uh $hn $ch $tx  ;  GShoPool $gm  ;  return 0
   }
   if {[set temp [lsearch -exact $gpool(-pls$gm) $nk]]=="-1"} {
     return 0
   } else {
     if {[llength $gpool(-pls$gm)]=="1"} {  set gpool(-pls$gm) ""
     } else {  set gpool(-pls$gm) [lreplace $gpool(-pls$gm) $temp $temp]  }
     GShoPool $gm
   }
 }
}

proc GStart {gm} {  global gpool  ;  set pls $gpool(-pls$gm)  ;  set t1ls ""  ;  set t2ls ""
 set pre "6\[[string toupper $gm]-Pool\]"  ;  set ch $gpool(chan)
 puthelp "PRIVMSG $ch :$pre14 Pool is full. Assigning teams...."
 while {[llength $pls]>"1"} {  set next [rand [llength $pls]]
  if {[GIsOdd [llength $pls]]} {  lappend t2ls [lindex $pls $next]
  } else {  lappend t1ls [lindex $pls $next]  }
  set pls [lreplace $pls $next $next]
 }
 lappend t2ls [lindex $pls 0]
 set putthis "14 (( Players:7 $gpool(-psz$gm)/$gpool(-psz$gm)14 )) - (( Team 1:"
 foreach nk $t1ls {  append putthis " 12$nk"
  if {$nk eq [lindex $t1ls end]} {  break  }
  append putthis "14 -"
 }
 append putthis "14 Team 2:"
 foreach nk $t2ls {  append putthis " 4$nk"
  if {$nk eq [lindex $t2ls end]} {  break  }
  append putthis "14 -"
 }
 puthelp "PRIVMSG $ch :$pre$putthis14 )) - (( Admin: 7$gpool(-admin$gm)14 ))"
 puthelp "PRIVMSG $ch :$pre14 Pool is now starting. Please wait for information. Your admin is 7$gpool(-admin$gm)14."
 set gpool(-info$gm) [list $gpool(-admin$gm) [concat $t1ls $t2ls] $gpool(-srvr$gm) $gpool(-pass$gm) $gpool(-onj$gm) 0]
 GClear $gm
 GSendInfo $gm 0
}

proc GSendInfo {gm start} {  global gpool botnick
 if {$gpool(-info$gm) eq ""} {  return 0  }
 set pre "6\[[string toupper $gm]-Pool\]"
 foreach {adm pls ip pas onj savd} $gpool(-info$gm) {  break  }
 set gip $ip  ;  set gpas $pas
 if {$ip ne "" && $pas ne ""} {  set do end
 } else {  set do 1
   if {$ip eq ""} {  set gip "undefined - pm your admin."  }
   if {$pas eq ""} {  set gpas "undefined - pm your admin."  }
 }
 set ln [join [lrange $gpool(msg1) $start $do]]
 set half [expr {[llength $pls]/2}]  ;  set idx 0  ;  set team 1  ;  set gf [GFullName $gm]
 foreach nik $pls {
  if {$idx==$half} {  incr team  }
  puthelp "PRIVMSG $nik :$pre14 [string map [list %g $gf %a $adm %p $gpool(pubpre) %s $gm %t $team %i $gip %w $gpas] $ln]"
  incr idx
 }
 if {$do eq "1"} {  set put "NOTICE $adm :\[[string toupper $gm]-Pool\]"
   if {$ip eq ""} {  puthelp "$put Set a ServerIP Now. Use: /MSG $botnick !ip <serverip:port>"  }
   if {$pas eq ""} {  puthelp "$put Set a Password Now. Use: /MSG $botnick !pass <password>"  }
 } else {
   if {$savd=="0"} {  set put "IP and password have been sent"
     incr savd  ;  set gpool(-info$gm) [list $adm $pls $ip $pas $onj $savd]
   } else {  set put "New IP and password have been sent"  }
   puthelp "PRIVMSG $gpool(chan) :$pre14 $put to every player. Contact 7$adm14 if you did not receive it."
 }
}

proc GShoPool {gm} {  global gpool  ;  set placnt 0
 set putthis "14 (( Players:7 [llength $gpool(-pls$gm)]/$gpool(-psz$gm)14 )) - (( Pool:"
 while {$placnt < $gpool(-psz$gm)} {  incr placnt
  if {$placnt > [llength $gpool(-pls$gm)]} {  append putthis " 3?"
  } else {  append putthis " 3[lindex $gpool(-pls$gm) [expr {$placnt-1}]]"  }
  if {$placnt < $gpool(-psz$gm)} {  append putthis "14 -"  } else {  break  }
 }
 puthelp "PRIVMSG $gpool(chan) :6\[[string toupper $gm]-Pool\]$putthis14 )) - (( Admin: 7$gpool(-admin$gm)14 ))"
}

putlog "10Game-Pool v.1.beta3 by 10SpiKe^^ Loaded."

array unset gtmp
